import 'dart:mirrors';

class Davison {

  static writeObjectJson(obj, StringBuffer sb) {
    
    final instanceMirror = reflect(obj);
    final cm = instanceMirror.type;

    sb.write('{');
    for (var member in cm.declarations.values) {
      
      if (member is VariableMirror) {
        var key = MirrorSystem.getName(member.simpleName);
        sb.write('"$key"');
        sb.write(':');
        var symbol = MirrorSystem.getSymbol(key);
        var value = instanceMirror.getField(symbol).reflectee;
        sb.write('"$value"');
        sb.write(',');
      }
    }
    sb.write('}');
  }

  static String toJson(obj) {
    
    final sb = StringBuffer();

    if (obj is List) {
      sb.write('[');
      obj.forEach((element) {
        writeObjectJson(element, sb);
        sb.write(',');
      });
    } else {
      writeObjectJson(obj, sb);
    }
    if(obj is List) {
      sb.write(']');
    }
    return sb.toString().replaceAll(',}', '}').replaceAll(',]', ']');
  }

  static createObjectFromJson(ClassMirror objectType, String json) {

    if (json[0] == '{') json = json.replaceFirst('{', '');
    if (json[json.length-1] == '}') json = json.substring(0, json.length-1);

    var object = objectType.newInstance(Symbol(''), []);

    var pairs = json.split(",");

    pairs.forEach((pair) {
      var keyOrValue = pair.split(':');
      var key = trimDoubleQoutes(keyOrValue[0]);
      
      dynamic value = keyOrValue[1];
      for (int i = 2; i < keyOrValue.length; i++) {
        value += keyOrValue[i];
      }
      value = trimDoubleQoutes(value);
      
      VariableMirror vm = objectType.declarations[Symbol(key)];
      var typeName = vm.type.simpleName.toString();

      if (typeName.contains('DateTime')) {
        value = DateTime.parse(value);
    }
      object.setField(Symbol(key), value);
    });
  
    return object.reflectee;
  }

  static T fromJson<T>(String json) {

    if (json.startsWith('[')) {
      final objectStartIndices = List<int>();
      final objectEndIndices = List<int>();

      ClassMirror listCm = reflectType(T);
      var listInstance = listCm.newInstance(Symbol(''), []);

      var elementType = listCm.typeArguments.first.reflectedType;
      ClassMirror elementCm = reflectType(elementType); 

      for (var i=0; i < json.length; i++) {
        if (json[i] == '{') {
          objectStartIndices.add(i);
        } else if (json[i] == '}') {
          objectEndIndices.add(i);
        }
      }

      if (objectStartIndices.length != objectEndIndices.length) {
        throw Exception('object index error');
      }

      for (var i=0; i < objectStartIndices.length; i++) {
        var element = createObjectFromJson(elementCm, json.substring(objectStartIndices[i], objectEndIndices[i]+1));
        listInstance.invoke(Symbol('add'), [element]);
      }

      return listInstance.reflectee;
    } else {
      final objectType = reflectType(T);
      final object = createObjectFromJson(objectType, json);
      return object;
    }
  }

  static String trimDoubleQoutes(String str) {
    var result = str.trim();
    if (result[0] == '"') {
      result = result.substring(1, result.length);
    }
    if(result[result.length-1] == '"') {
      result = result.substring(0, result.length-1);
    }
    return result;
  }
}